/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.app.service;

import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.dto.AdvancedQueryOptions;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.scriptcase.dto.app.*;

import java.util.List;

/**
 * 应用表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface AppApiService extends ExcelService
{
  /**
   * 查询应用表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<AppDetailDto> pagingQuery(QueryByPage<AppQueryDto> queryByPage);

  /**
   * 新增应用表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(AppAddDto dto);

  /**
   * 更新应用表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(AppUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取应用表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  AppDetailDto findByPK(String uuid);

  /**
   * 删除应用表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<AppAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<AppUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  AppDetailDto getAppDetail(String appId);


  /**
   * 根据id集合查询app信息
   * @param ids id集合
   * @return 结果集
   */
  List<AppDetailDto> queryByIds(List<String> ids);

  /**
   * 分页查询app版本详情
   * @param queryByPage 查询参数
   * @return 结果集
   */
    Pagination<AtuAppVersionDto> pagingQueryAppVersions(QueryByPage<AppVersionQueryDto> queryByPage);

    AdvancedQueryOptions advancedQueryOptions(String projectId);
}
