/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2023 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.caseDataSet.service;

import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.dto.AdvancedQueryOptions;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.scriptcase.dto.IdAndStatusDto;
import net.northking.cctp.scriptcase.dto.atuScript.atuScriptInfo.PrincipalUpdateDto;
import net.northking.cctp.scriptcase.dto.caseDataSet.*;

import java.util.List;

/**
 * 用例集目录表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface AtuCaseDataSetApiService extends ExcelService {
  /**
   * 查询用例集目录表列表
   *
   * @param queryByPage 查询条件
   * @return 查询结果列表
   */
  Pagination<AtuCaseDataSetDetailDto> pagingQuery(QueryByPage<AtuCaseDataSetQueryDto> queryByPage);

  /**
   * 新增用例集目录表
   *
   * @param dto 接口数据传输对象
   * @return 主键
   */
  String add(AtuCaseDataSetAddDto dto);

  /**
   * 更新用例集目录表
   *
   * @param dto 接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(AtuCaseDataSetUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取用例集目录表
   *
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  AtuCaseDataSetDetailDto findByPK(String uuid);

  /**
   * 删除用例集目录表
   *
   * @param uuid 主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

  /**
   * 批量新增
   *
   * @param batch 集合
   * @return 成功的数量
   */
  List<String> addBatch(List<AtuCaseDataSetAddDto> batch);

  /**
   * 批量更新
   *
   * @param batch 集合
   * @return 成功的数量
   */
  Integer updateBatch(List<AtuCaseDataSetUpdateDto> batch) throws IllegalAccessException;

  /**
   * 批量删除
   *
   * @param batch 集合
   * @return 成功的数量
   */
  Integer deleteBatch(List<Object> batch);

  List<AtuCaseDataSetMenuDto> querySetTree(AtuSetTreeQueryDto dto);

  List<AtuCaseDataSetMenuDto> queryPlanSetTree(AtuCaseDataSetFeignDto dto);

  void saveScriptSetTree(SetTreeDto dto);

  void saveSceneSetTree(SetTreeDto dto);

  Pagination<AtuCaseScriptResultDto> queryScriptByPath(QueryByPage<AtuCaseScriptQueryDto> dto);

  Integer updatePrincipal(PrincipalUpdateDto dto);

  Pagination<AtuCaseScriptResultDto> queryScriptByPlan(QueryByPage<AtuCaseScriptQueryDto> dto);

  List<AtuPlanScriptReturnDto> feignQueryScriptId(AtuPlanScriptSaveDto dto);


  /**
   * 啄木鸟查用例集目录树
   * @param dto
   * @return
   */
  List<AtuCaseDataSetMenuDto> picusQuerySetTree(PicusQueryDto dto);

  /**
   * 根据Id查看脚本状态
   * @param ids
   * @return
   */
  List<IdAndStatusDto> picusQueryByScriptIds(List<String> ids);

  List<String> queryScriptPlatforms(String planId);
    /**
     * 获取高级查询筛选项
     * @return
     */
    AdvancedQueryOptions advancedQueryOptions(String projectId);

    AdvancedQueryOptions picusAdvancedQueryOptions(String projectId);
}
