/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.device.api.model;

import cn.gjing.tools.auth.annotation.RequiredPermissions;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.http.ResultWrapper;

import net.northking.cctp.device.api.model.service.CdDeviceModelApiService;
import net.northking.cctp.device.dto.model.CdDeviceModelAddDto;
import net.northking.cctp.device.dto.model.CdDeviceModelDetailDto;
import net.northking.cctp.device.dto.model.CdDeviceModelUpdateDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 设备型号管理表 => Restful 类型 HTTP/HTTPS 接口
 *
 * @author maven-cctp-plugin
 * @since 1.0
 */
@Api(tags = "设备型号管理表")
@RequestMapping("/v1/models")
@RestController
public class CdDeviceModelRestfulCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(CdDeviceModelRestfulCtrl.class);

  @Autowired
  private CdDeviceModelApiService apiService;

  /**
   * Restful接口-post： 新增设备型号管理表
   *
   * @param inputDto         接口数据传输对象
   * @return 数据库主键
   */
  @ApiOperation(value = "新增：设备型号管理表")
  @PostMapping(
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<String> add(
      @RequestBody @Validated CdDeviceModelAddDto inputDto)
  {
    ResultWrapper<String> wrapper = new ResultWrapper<>();
    String uuid = apiService.add(inputDto);
    return wrapper.success(uuid);
  }

  /**
   * Restful接口-put： 更新设备型号管理表
   *
   * @param inputDto         接口数据传输对象
   * @return 修改的记录数量
   */
  @ApiOperation(value = "更新：设备型号管理表")
  @RequiredPermissions({"ui-updateModels","ui-updateModels2"})
  @PutMapping(
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> update(
      @RequestBody @Validated CdDeviceModelUpdateDto inputDto) throws IllegalAccessException
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = apiService.updateByPK(inputDto);
    return wrapper.success(count);
  }

  /**
   * Restful接口-get： 获取设备型号管理表
   *
   * @param uuid       数据库主键
   * @return 接口数据传输对象
   */
  @ApiOperation(value = "获取：设备型号管理表")
  @GetMapping(
      value = "/{uuid}",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<CdDeviceModelDetailDto> findByPK(
      @PathVariable("uuid") String uuid)
  {
    ResultWrapper<CdDeviceModelDetailDto> wrapper = new ResultWrapper<>();
    CdDeviceModelDetailDto user = apiService.findByPK(uuid);
    return wrapper.success(user);
  }

  /**
   * Restful接口-delete： 删除设备型号管理表
   *
   * @param uuid       数据库主键
   * @return 删除的记录数量
   */
  @ApiOperation(value = "删除：设备型号管理表")
  @DeleteMapping(
      value = "/{uuid}",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> deleteByPK(
      @PathVariable("uuid") String uuid)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = apiService.deleteByPK(uuid);
    return wrapper.success(count);
  }
}
