/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.testbug.api.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.testbug.dto.BugStatisticsChartAddDto;
import net.northking.cctp.testbug.dto.BugStatisticsChartDetailDto;
import net.northking.cctp.testbug.dto.BugStatisticsChartQueryDto;
import net.northking.cctp.testbug.dto.BugStatisticsChartUpdateDto;

import java.util.List;

/**
 * 状态统计Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface BugStatisticsChartApiService extends ExcelService
{
  /**
   * 查询状态统计列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<BugStatisticsChartDetailDto> pagingQuery(QueryByPage<BugStatisticsChartQueryDto> queryByPage);

  /**
   * 新增状态统计
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(BugStatisticsChartAddDto dto);

  /**
   * 更新状态统计
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(BugStatisticsChartUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取状态统计
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  BugStatisticsChartDetailDto findByPK(String uuid);

  /**
   * 删除状态统计
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<BugStatisticsChartAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<BugStatisticsChartUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    int deleteAll();
}
